const { Console } = require('console')
const queue = require('./queue')

class updater {
    constructor(){
        this.queue = queue        
        this.environment = require('./environment')
        this.identification = this.environment.identification;

        this.fs = require('fs')
        this.path = require('path')
        this.doneFilePath = this.path.join(process.cwd(), 'done.json')
        this.doingFilePath = this.path.join(process.cwd(), 'doing.json')

        if(this.fs.existsSync(this.doneFilePath)){
            try {
                this.queue.done = JSON.parse(this.fs.readFileSync(this.doneFilePath))
            } catch( e ) {
                this.queue.done = []
            }
        } else {
            this.saveDone();
        }

        if(this.fs.existsSync(this.doingFilePath)){
            try {
                this.queue.doing = JSON.parse(this.fs.readFileSync(this.doingFilePath))
            } catch ( e ){
                this.queue.doing = []
            }
            
        } else {
            this.saveDoing();
        }

    }

    saveDoing(){
        this.fs.writeFileSync(this.doingFilePath, JSON.stringify(this.queue.doing))
    }

    saveDone(){
        this.fs.writeFileSync(this.doneFilePath, JSON.stringify(this.queue.done))
    }

    setCurrentObjectDone(currentObjectDone){
        //Handle changes
        let addDone = []
        let removeDone = []
        
        // current object - done
        let curObj = this.queue.done

        // New current object - done
        let newObj = JSON.stringify(currentObjectDone)
        newObj = JSON.parse( newObj )
        newObj = !newObj ? {} : newObj;

        // Put on removeDone
        Object.keys( curObj ).forEach( keyFilial => {
            let filial = curObj[keyFilial]
            Object.keys( filial ).forEach( keyLoja => {
                let loja = filial[keyLoja]
                if(newObj[keyFilial] == undefined){
                    newObj[keyFilial] = {}
                }

                if(newObj[keyFilial][keyLoja] == undefined){
                    newObj[keyFilial][keyLoja] = {}
                }
                
                Object.keys( loja ).forEach( keySeqVenda  => {
                    if ( newObj[keyFilial][keyLoja][keySeqVenda] == undefined ){
                        let remObj = {}
                        if ( !remObj[keyFilial] ) { remObj[keyFilial] = {} }
                        if ( !remObj[keyFilial][keyLoja] ) { remObj[keyFilial][keyLoja] = {} }
                        if ( !remObj[keyFilial][keyLoja][keySeqVenda] ) { remObj[keyFilial][keyLoja][keySeqVenda] = {} }
                        remObj[keyFilial][keyLoja][keySeqVenda] = JSON.stringify(curObj[keyFilial][keyLoja][keySeqVenda])
                        remObj[keyFilial][keyLoja][keySeqVenda] = JSON.parse( remObj[keyFilial][keyLoja][keySeqVenda] )
                        removeDone.push( remObj )
                    } else {
                        //If data change remove and add again
                        if( newObj[keyFilial][keyLoja][keySeqVenda].DTNOTIFICACAO !== curObj[keyFilial][keyLoja][keySeqVenda].DTNOTIFICACAO){
                            let remObj = {}
                            if ( !remObj[keyFilial] ) { remObj[keyFilial] = {} }
                            if ( !remObj[keyFilial][keyLoja] ) { remObj[keyFilial][keyLoja] = {} }
                            if ( !remObj[keyFilial][keyLoja][keySeqVenda] ) { remObj[keyFilial][keyLoja][keySeqVenda] = {} }
                            remObj[keyFilial][keyLoja][keySeqVenda] = JSON.stringify(newObj[keyFilial][keyLoja][keySeqVenda])
                            remObj[keyFilial][keyLoja][keySeqVenda] = JSON.parse( remObj[keyFilial][keyLoja][keySeqVenda] )
                            removeDone.push( remObj )
                            addDone.push( remObj)
                        }
                        delete newObj[keyFilial][keyLoja][keySeqVenda]
                    }
                })
            })
        })

        // Put on addDone
        Object.keys( newObj ).forEach( keyFilial => {
            let filial = newObj[keyFilial]
            Object.keys( filial ).forEach( keyLoja => {
                let loja = filial[keyLoja]
                Object.keys( loja ).forEach( keySeqVenda  => {
                    let addObj = {}
                    if ( !addObj[keyFilial] ) { addObj[keyFilial] = {} }
                    if ( !addObj[keyFilial][keyLoja] ) { addObj[keyFilial][keyLoja] = {} }
                    if ( !addObj[keyFilial][keyLoja][keySeqVenda] ) { addObj[keyFilial][keyLoja][keySeqVenda] = {} }

                    addObj[keyFilial][keyLoja][keySeqVenda] = JSON.stringify( newObj[keyFilial][keyLoja][keySeqVenda] )
                    addObj[keyFilial][keyLoja][keySeqVenda] = JSON.parse( addObj[keyFilial][keyLoja][keySeqVenda] )

                    addDone.push( addObj )
                })
            })
        })

        if (Object.keys( removeDone ).length > 0) {
            this.socket.emit('removeDone', removeDone)
            if( this.environment.publisher_api.use_publisher == true){
                this.queue.send({'removeDone': removeDone})
            }

        }

        if (Object.keys( addDone ).length > 0) {
            this.socket.emit('addDone', addDone)
            if( this.environment.publisher_api.use_publisher == true){
                this.queue.send({'addDone':addDone})
            }
        }

        this.queue.done = currentObjectDone
        this.saveDone()
    }

    getCurrentObjectDone(){
        return this.queue.done
    }

    setCurrentObjectDoing(currentObjectDoing){
        //Handle changes for doing
        let addDoing = []
        let removeDoing = []

        // Current object - doing
        let curObj = this.queue.doing

        // New current object - doing
        let newObj = JSON.stringify(currentObjectDoing)
        newObj = JSON.parse( newObj )

        // Put on removeDoing
        Object.keys( curObj ).forEach( keyFilial => {
            let filial = curObj[keyFilial]
            Object.keys( filial ).forEach( keyLoja => {
                let loja = filial[keyLoja]
                if(newObj[keyFilial] == undefined){
                    newObj[keyFilial] = {}
                    newObj[keyFilial][keyLoja] = {}
                }
                Object.keys( loja ).forEach( keySeqVenda  => {
                    if ( newObj[keyFilial][keyLoja][keySeqVenda] == undefined ){
                        let remObj = {}
                        if ( !remObj[keyFilial] ) { remObj[keyFilial] = {} }
                        if ( !remObj[keyFilial][keyLoja] ) { remObj[keyFilial][keyLoja] = {} }
                        if ( !remObj[keyFilial][keyLoja][keySeqVenda] ) { remObj[keyFilial][keyLoja][keySeqVenda] = {} }
                        remObj[keyFilial][keyLoja][keySeqVenda] = JSON.stringify(curObj[keyFilial][keyLoja][keySeqVenda])
                        remObj[keyFilial][keyLoja][keySeqVenda] = JSON.parse( remObj[keyFilial][keyLoja][keySeqVenda] )
                        removeDoing.push( remObj )

                    } else {
                        delete newObj[keyFilial][keyLoja][keySeqVenda]
                    }
                })
            })
        })

        // Put on addDoing
        Object.keys( newObj ).forEach( keyFilial => {
            let filial = newObj[keyFilial]
            Object.keys( filial ).forEach( keyLoja => {
                let loja = filial[keyLoja]
                Object.keys( loja ).forEach( keySeqVenda  => {
                    let addObj = {}
                    if ( !addObj[keyFilial] ) { addObj[keyFilial] = {} }
                    if ( !addObj[keyFilial][keyLoja] ) { addObj[keyFilial][keyLoja] = {} }
                    if ( !addObj[keyFilial][keyLoja][keySeqVenda] ) { addObj[keyFilial][keyLoja][keySeqVenda] = {} }

                    addObj[keyFilial][keyLoja][keySeqVenda] = JSON.stringify( newObj[keyFilial][keyLoja][keySeqVenda] )
                    addObj[keyFilial][keyLoja][keySeqVenda] = JSON.parse( addObj[keyFilial][keyLoja][keySeqVenda] )

                    addDoing.push( addObj )
                })
            })
        })

        if (Object.keys( removeDoing ).length > 0) {
            this.socket.emit('removeDoing', removeDoing)
            if( this.environment.publisher_api.use_publisher == true){
                this.queue.send({'removeDoing':removeDoing})
                this.queue.send({'addDone':removeDoing})
                this.queue.send({'removeDone':removeDoing})
            }
        }

        if (Object.keys( addDoing ).length > 0) {
            this.socket.emit('addDoing', addDoing)
            if( this.environment.publisher_api.use_publisher == true){
                this.queue.send({'addDoing': addDoing})
            }
        }

        this.queue.doing = currentObjectDoing
        this.saveDoing()
    }

    getCurrentObjectDoing(currentObjectDoing){
        return this.queue.doing
    }

    setSocket(socket){
        this.socket = socket;
    }

    updateDoing( newDoing ){
        let newDoingProcessed = {};
        newDoing.forEach(function(item, key, arr){
            if ( newDoingProcessed[item['CDFILIAL']] == undefined ){
                newDoingProcessed[item['CDFILIAL']] = {}
            }
            if ( newDoingProcessed[item['CDFILIAL']][item['CDLOJA']] == undefined){
                newDoingProcessed[item['CDFILIAL']][item['CDLOJA']] = {}
            }
            if ( newDoingProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRPEDIDOFOS']] == undefined){
                newDoingProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRPEDIDOFOS']] = {}
            }
            newDoingProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRPEDIDOFOS']]["CDSENHAPED"] = item['CDSENHAPED']
            newDoingProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRPEDIDOFOS']]["NRVENDAREST"] = item['NRVENDAREST']
            newDoingProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRPEDIDOFOS']]["IDORIGEMVENDA"] = item['IDORIGEMVENDA']
        });
        this.setCurrentObjectDoing(newDoingProcessed)
    }

    updateDone( newDone ){
        let newDoneProcessed = {};
        newDone.forEach(function(item, key, arr){
            if ( newDoneProcessed[item['CDFILIAL']] == undefined ){
                newDoneProcessed[item['CDFILIAL']] = {}
            }
            if ( newDoneProcessed[item['CDFILIAL']][item['CDLOJA']] == undefined){
                newDoneProcessed[item['CDFILIAL']][item['CDLOJA']] = {}
            }
            if ( newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']] == undefined){
                newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']] = {}
            }
            newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']]["CDSENHAPED"] = item['CDSENHAPED']
            newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']]["NRVENDAREST"] = item['NRVENDAREST']
            newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']]["IDORIGEMVENDA"] = item['IDORIGEMVENDA']
            newDoneProcessed[item['CDFILIAL']][item['CDLOJA']][item['NRSEQVENDA']]["DTNOTIFICACAO"] = item['DTNOTIFICACAO'].toJSON()
        });
        this.setCurrentObjectDone(newDoneProcessed);
    }
}

module.exports = new updater();